@@ -1,6 +1,5 @@
 from django.apps import AppConfig
 
-from .config import settings
 from .task import auto_convert_commands_to_tasks
 
 
@@ -11,4 +10,4 @@ class DjangoCeleryExtensionsAppConfig(AppConfig):
 
     def ready(self):
         auto_convert_commands_to_tasks()
-        from django_celery_extensions.checks import check_celery_tasks
+        from django_celery_extensions.checks import check_celery_tasks  # noqa: F401
